<?php

namespace App\Http\Controllers;

use Illuminate\Support\Facades\Http;
use Illuminate\Http\Request;
use phpseclib\Crypt\RSA;

class SudaniSubscription extends Controller
{
    protected $baseURL;
    protected $password;
    protected $username;
    protected $providerKey;
    protected $serviceCode;
    protected $_client;

    protected $dashboardServiceCode;

    public function __construct()
    {
        $this->baseURL =env('SUDANI_SERVICE_URL');
        $this->password =env('SUDANI_PASSWORD');
        $this->username =env('SUDANI_USER_NAME');
        $this->providerKey =env('SUDANI_PROVIDER_KEY');
        $this->serviceCode =env('MIX_SUDANI_SERVICE_CODE');
        $this->dashboardServiceCode=env('DASHBOARD_SERVICE_CODE');
        // $this->validateConfigs();
        $this->_client = new \GuzzleHttp\Client();
    }

    /**
     * Login to SPay
     *
     * @return array
     */
    public function login()
    {
       
        $response = Http::post($this->baseURL . 'Login/', [
            'login' => $this->username,
            'password' => $this->getEncryptedPassword(),
        ]);

        $body = json_decode($response->getBody(), true);

      

        return $body;
    }

    /**
     * Get public key
     *
     * @return array
     */
    public function getPublicKey()
    {
        $response = Http::post($this->baseURL . 'GetPublicKey/', [
            'providerKey' => $this->providerKey,
        ]);

        $body = json_decode($response->getBody(), true);


        return $body;
    }

    /**
     * Initial subscription
     *
     * @param string $phone
     * @return array
     */
    public function initiate($phone)
    {
      

        $response = Http::withHeaders($this->getHeaders())
            ->post($this->baseURL . 'InitPay/', [
                'msisdn' => $phone,
                'serviceCode' => $this->serviceCode,
            ]);

        $body = json_decode($response->getBody(), true);

      

        return $body;
    }

    /**
     * Verify OTP and complete payment/subscription
     *
     * @param string $otp
     * @param string $requestId
     * @return array
     */
    public function verify($otp, $requestId)
    {
     

        $response = Http::withHeaders($this->getHeaders())
            ->post($this->baseURL . 'Payment/', [
                'pin' => $otp,
                'requestId' => $requestId,
            ]);

        $body = json_decode($response->getBody(), true);

        return $body;
    }

    /**
     * Check subscription status of a single phone
     *
     * @param string $phone
     * @return array
     */
    public function checkSubscription($phone)
    {
     

        $response = Http::withHeaders($this->getHeaders())
            ->post($this->baseURL . 'CheckSubscription/', [
                'msisdn' => $phone,
                'serviceCode' => $this->serviceCode,
            ]);

        $body = json_decode($response->getBody(), true);

     

        return $body;
    }

    /**
     * Unsubscribe specific phone
     *
     * @param string $phone
     * @return array
     */
    public function unsubscribe($phone)
    {
      

        $response = Http::withHeaders($this->getHeaders())
            ->post($this->baseURL . 'UnSubscribe/', [
                'msisdn' => $phone,
                'serviceCode' => $this->serviceCode,
            ]);

        $body = json_decode($response->getBody(), true);

       
        return $body;
    }

    /**
     * Get headers
     *
     * @return array
     */
    private function getHeaders()
    {
        return [
            'Content-Type' => 'application/json',
            'token' => $this->getAvailableToken(),
        ];
    }

    /**
     * Get SPay token
     *
     * @return string
     */
    public function token()
    {
       
        $response = $this->login();

        if (isset($response['status']) && $response['status'] === true) {

          

            $this->token = $response['token']; // To be cached into this instance

            return $response['token'];
        }

        return null;
    }

    /**
     * Get available SPay token, from loacl or remote
     *
     * @return string
     */
    private function getAvailableToken()
    {
        return $this->token ?? $this->token();
    }

    /**
     * Provide ready local SPay token
     *
     * @return AlkhatibDev\LaravelSudani\Sudani
     */
    public function withToken($token)
    {
    

        $this->token = $token;

        return $this;
    }

    /**
     * Validate config file and its values
     *
     * @return void
     */
    public function validateConfigs()
    {
        

        if (
            is_null($this->baseURL) ||
            is_null($this->password) ||
            is_null($this->username) ||
            is_null($this->providerKey) ||
            is_null($this->serviceCode)
        ) {
            $message = __('The provided configs is invalid, make sure laravel-sudani config file is published and all its configs are set.');

           

            throw new InvlalidConfigsValuesException($message);
        }
    }

    /**
     * Encrypt password using public key
     *
     * @param string $publicKey
     * @return string
     */
    public function encryptPassword($publicKey)
    {
        
        $key = $this->getRSApublicKey($publicKey);

        return base64_encode(
            $this->encrypt($this->password, $key)
        );
    }

    /**
     * Get encrypted password text
     *
     * @return string
     */
    public function getEncryptedPassword()
    {
        $response = $this->getPublicKey();

        if ($response['status'] === true) {
            
            return $this->encryptPassword($response['publicKey']);
        }

        return null;
    }


     public function getRSApublicKey($publicKey) {
       // return " ok";
        $rsa = new RSA();
       
        $rsa->loadKey($publicKey);
        $rsa->setPublicKey($publicKey);
        return $rsa->getPublicKey();
    }

    /**
     * Encrypt a message
     *
     * @param $message - message to encrypt
     * @param $key - encryption key
     * @return string
     */
    static function encrypt($message, $key) {
        openssl_public_encrypt($message, $encrypted, $key, OPENSSL_PKCS1_PADDING);
        return $encrypted;
    }

    /**
     * Decrypt a message
     * @param $encrypted - message encrypted with safeEncrypt()
     * @param $key - key used for encryption
     * @return string
     */
    static function decrypt($encrypted, $key) {
        openssl_public_decrypt($encrypted, $plain, $key);
        return $plain;
    }

    public function Dashboard($phone, $status){
        $_client = new \GuzzleHttp\Client();
        $response = $_client->request( "POST",
        '164.92.184.24:81/SubscriptionLog/CreatefromAPI',
        [
            'headers' => [
                "Accept" =>  "application/json",
                "Content-Type" =>  "application/json"
            ],
            'json' => [
                'subscriberPhone' => $phone,
                'serviceId' => $this->dashboardServiceCode,
                'status'=>$status
                ]
        ]
    );
    return $response;
     }
}
